/***************************************************************************
                          dx7_voice.cpp  -  description
                             -------------------
    begin                : Mon Jan 28 2002
    copyright            : (C) 2002 by Juan Linietsky
    email                : coding@reduz.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dx7_voice.h"
#include <math.h>

Sint32 Dx7_Voice::sin_table[SINE_LENGTH];
Sint32 Dx7_Voice::eg_amp_table[ENV_LENGTH];
bool Dx7_Voice::tables_created=false;


void Dx7_Voice::calculate_op(Op& p_op) {
	
	float mult;

		
	mult = sin_table[ ((p_op.phase_count>>FRAC_TO_SINETABLE_SHIFT) + (*p_op.in)) & SINE_MASK];		
		
	mult*=Dx7_Tables::EG_amp_to_peak[p_op.eg.get_val()];
		
	p_op.selfout=mult;
		
	*p_op.out+=p_op.selfout;
}

void Dx7_Voice::mix_internal(int p_amount,Sint32* p_where) {

	int i;
	double ffeedback;
        if (status==DEAD) return;


        while (p_amount--) {

		joint.out=0;	
	
		ffeedback=*joint.feedback_ptr;
		ffeedback*=ffeedback_multiplier;
	

		joint.feedback=ffeedback;
	
		joint.inter[0]=0;
		joint.inter[1]=0;
		joint.inter[2]=0;
		joint.inter[3]=0;
		joint.inter[4]=0;
		joint.inter[5]=0;
	
		for (i=5;i>=0;i--) {
			calculate_op(op[i]);
			op[i].increment();
		}
		
  		*p_where++ +=joint.out << 12;
  		*p_where++ +=joint.out << 12;
  	}
}


void Dx7_Voice::init_tables() {

	if (tables_created) return;

	signed int i,x;
	signed int n;
	double o,m;



	for (i=0; i<SINE_LENGTH; i++) {
	
		double sine_val;
		
		sine_val=sin( ((i)*(2*M_PI)) / SINE_LENGTH ); // index * radian cycle / index max
		sine_val*=SINE_LENGTH/(2*M_PI); //quite a big table yes		
		sin_table[i]=sine_val;
	}
	

	
	for (i=0; i<ENV_LENGTH; i++) {

       		double multiplier;
			
       		multiplier=Dx7_Tables::EG_amp_to_peak[i];
       		multiplier*=MATRIX_FRAC_LENGTH;
       		eg_amp_table[i]=multiplier;
	}			
	
//	i/=0;

	tables_created=true;	
}


void Dx7_Voice::set_mix_frequency_internal(int p_mixfreq) {

	base_freq=p_mixfreq;
 	for (int i=0;i<MAX_OPERATORS;i++) {

		op[i].eg.base_freq=base_freq;
		op[i].base_freq=base_freq;
  	}          	
}

Dx7_Voice::Dx7_Voice(){
        set_mix_frequency(44100);
	joint.feedback=0;
	joint.feedback_ptr=&joint.feedback;	
	
	for (int i=0;i<MAX_OPERATORS;i++) {
		op[i].eg.index=op[i].index=i+1;

	}	
	
	channel_in_use=-1;
	
	init_tables();
	status=DEAD;

	
}
Dx7_Voice::~Dx7_Voice(){


}
