/***************************************************************************
                          dx7_voice.h  -  description
                             -------------------
    begin                : Mon Jan 28 2002
    copyright            : (C) 2002 by Juan Linietsky
    email                : coding@reduz.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DX7_VOICE_H
#define DX7_VOICE_H

#include "typedefs.h"
#include "dx7_tables.h"
#include "voice.h"
#include "math.h"
/**
  *@author Juan Linietsky
  */

#define SUM_BITS Sint32
//#define INTERPOLATE

class Dx7_Voice : public Voice {
public:


		
       	struct Data {
	
		struct Envelope {
		
			int rate[4];
			int level[4];
				
		};
	
		struct Operator {
		
			Envelope envelope;		

	                bool enable;				
			int kb_scale_bkpoint;
			int kb_scaling_l_depth;
			int kb_scaling_r_depth;
			int kb_scaling_l_curve;
			int kb_scaling_r_curve;
			
			int rate_scale;
			int detune;
			int amp_modulation_sens;
			int key_vel_sens;
			
			int output_level;
			int osc_mode;
			int coarse_tune;
			int fine_tune;
			
	
		} operators[6];
		
		
		Envelope pitch_envelope;
		
		
		int algorithm;
		int feedback_level;
		int oscilator_key_sync;
		
		struct LFO {
			
			int speed;
			int delay;
			int pitch_modulation_depth;
			int pitch_modulation_sens;
			int pitch_amplitude_depth;

			int sync;
			int wave;
		} lfo;
		
		int transpose;

  		
  		Data() {

            		memset(this,0,sizeof(Data));
            	}
	};
	
	int base_freq;

        Uint32 feedback_multiplier;
        float ffeedback_multiplier;
        Status status;
        char channel_in_use;
        char note_in_use;


        enum {


       		FRAC_SHIFT=18,
       		FRAC_LENGTH=(1<<FRAC_SHIFT),
       		FRAC_MASK=(FRAC_LENGTH-1),

       		ENV_LENGTH=100,
       		LEV_LENGTH=100,
       		
		SINE_BITS=12,
		SINE_LENGTH=(1<<SINE_BITS),
        	SINE_MASK=(SINE_LENGTH-1),
        	
        	MATRIX_FRAC_SHIFT=18,
        	MATRIX_FRAC_LENGTH=(1<<MATRIX_FRAC_SHIFT),
        	
        	
        	AMP_BITS=(SINE_BITS),
        	AMP_LENGTH=(1<<AMP_BITS),
        	AMP_MASK=(AMP_LENGTH-1),
        	
       		MODULATION_BITS=(FRAC_SHIFT-SINE_BITS),		        	
        	FRAC_TO_SINETABLE_SHIFT=(FRAC_SHIFT-SINE_BITS),

        	FDBK_MULT_BITS=4,
        	FDBK_MULT_LENGTH=(1<<FDBK_MULT_BITS),   	
       	
        };


        enum Ops {

		OP_1=0,
		OP_2,
		OP_3,
		OP_4,
		OP_5,
		OP_6,
		MAX_OPERATORS
        };

       	struct Eg { //envelope generator
		
       		enum Phase {
       		
       			ATTACK,
       			SUSTAIN,
       			RELEASE,
       			FINISHED
       		};
	
       		Uint16 rate[4];
       	        Uint16 level[4];
				
                int index;
       	        Sint8 current_node;
       	        Sint32 val;
       	        Sint32 climb_rate;
       	        Phase phase;
                bool sustain_disable;
                int base_freq;

		void set_release_phase();
		void setup_next_node();
       	
       	        void set_climb_rate(Uint16 p_rate,Uint16 p_level_l,Uint16 p_level_r);
       	        void reset();
       	
       	        inline void process() {
       	        	
       	        	if ((phase==SUSTAIN) || (phase==FINISHED)) return;

       	        	
			val+=climb_rate;
		
			
			if (climb_rate>0) {
	
				if ((val>>FRAC_SHIFT)>level[current_node]) setup_next_node();
				
			} else {
	
				if ((val>>FRAC_SHIFT)<level[current_node]) setup_next_node();	
			}
       	        }
       	
       	        inline Uint16 get_val() {

			return val>>FRAC_SHIFT;
		}

                Eg();
       	};


	struct Op {

       	        Uint8 index;	
		
		Eg eg;

		Data::Envelope eg_base;
		
	        Uint32 velocity;
	        Uint32 velocity_sens;
	       	Uint32 phase_count;
		Uint32 phase_increment;
		Uint32 eg_level;
		Uint16 coarse;
		Uint32 fine;
		Uint32 freq;
		float freq_multiplier;
		Uint8 detune;
		int base_freq;

		SUM_BITS *in,*out; // where to take the input from, where to put the output
      		
      		SUM_BITS zero,selfout;

                Uint8 output_level;

		inline void increment() {
		
			phase_count+=phase_increment;
			eg.process();
		}
		
		
		void set_velocity(Uint8 p_velocity);		
		void set_velocity_sens(Uint8 p_velocity);		
		void set_coarse(Uint8 p_coarse);
		void set_fine(Uint8 p_detune);
		void set_detune(Uint8 p_detune);
		void set_freq(Uint32 p_freq);
		void set_freq_multiplier(float p_freq_mult);		
		void recalculate_increment();
		void set_output_level(Uint8 p_level);
		void prepare_envelope();
		void reset();
	
		
		Op();

	} op[MAX_OPERATORS];

 	void calculate_op(Op& p_op);	

        struct Joints {

		SUM_BITS zero; //take zero from here
		SUM_BITS feedback; //take feedback from here
		SUM_BITS *feedback_ptr; //point where feedback goes
		SUM_BITS inter[MAX_OPERATORS]; //interoperator joints
                SUM_BITS out;
                Sint32 out_feedback;

        } joint;


        void connect_op(int p_op_from,int p_op_to,int p_cn_through);
        void set_algorithm(int p_algo);
        void init_tables();
        void set_carrier(int p_op);
        void set_feedback(int p_op_from,int p_op_to);

	static Sint32 eg_amp_table[ENV_LENGTH];
	static Sint32 sin_table[SINE_LENGTH];

	
	static bool tables_created;

	
public:	
	

	Status get_status();
	
	void poll_for_dead_staus();	

        void set_note_internal(char p_note,char p_velocity);
	void set_note_off_internal(char p_velocity);
        void set_data(Data *p_data);

	void set_mix_frequency_internal(int p_mixfreq); //set mixing freq
	void mix_internal(int p_amount,Sint32* p_where); //set where to mix
	void recalculate_pitch_internal();
// 	void kill();
	

	Dx7_Voice();
	~Dx7_Voice();
};

#endif
