/***************************************************************************
                          sine_voice.cpp  -  description
                             -------------------
    begin                : Wed Jul 10 2002
    copyright            : (C) 2002 by red
    email                : red@server
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "sine_voice.h"
#include "math.h"

Voice::Status SINE_Voice::get_status(){ //obvious function

	return (decay==0) ? DEAD : ATTACKING; //i think i should change this to SUSTAINING.. ohwell..
	//dont return DEAD unless the voice finished, when you return DEAD, the voice is _killed_.	
}
void SINE_Voice::poll_for_dead_staus(){ //call in case we try to figure out if this note voice is dead

	//but we wont. i think i'll take this function out, it's just for compatibility with DX7
}
void SINE_Voice::set_note_internal(char p_note,char p_velocity) {
	//there is not really need to take the parameters unless you _really_ want them
        //the best is to just use get_fnote() and get_total_volume() later on
	
	recalculate_increment(); //we'll recalculate the increment

}

void SINE_Voice::set_note_off_internal(char p_velocity){

	noteoff_was_pressed=true; //noteoff was pressed!
}


void SINE_Voice::recalculate_increment() {


	float basenote;
	float frequency;

        basenote=get_fnote(); //fnote is the note, but given as FLOAT (0-127 with decimals), so it allows perfect control for vibratos/sweeps/etc

	basenote+=used_data.coarse; //apply coarse tunning

	frequency=8.1757989156*pow(2.0,basenote/12.0); //note -> frequency conversion  8.1757989156 is the midi C-0

	increment=frequency*M_PI*2;
	increment/=base_freq;
/*	
	"frecuency" incrementos son base freq

	freq*incr=base_freq;
	freq=base_freq/incr;*/

}

void SINE_Voice::set_data(Data *p_data) {

	used_data=*p_data;
}

void SINE_Voice::mix_modifier_call() {
	//remember, this function is called 50 times per second

	if (!noteoff_was_pressed) return;
	
	decay-=(float)(10-used_data.decay)/15.0+0.1; //little formula for a cute decay
	if (decay<0) decay=0;

}

void SINE_Voice::recalculate_pitch_internal() {

	recalculate_increment(); // let's just recalculate the increment
}


void SINE_Voice::set_mix_frequency_internal(int p_mixfreq){

	base_freq=p_mixfreq;
	recalculate_increment();

}

void SINE_Voice::mix_internal(int p_amount,Sint32* p_where){

	if (!used_data.play) return; //if we're not playing it.. notice, this property is only to show how a boolean property works, this behavior itself is actually stupid ;)

        float mix_volume_left;
        float mix_volume_right;
        float mix_volume;
        float val,left,right;
        
        mix_volume=get_total_volume(); //get_total_volume returns the TOTAL volume in a range from 0 to 1, it inclues main volume, preamp, expression, velocity, tremolo, etc
        mix_volume*=decay; //apply decay
        
	mix_volume_right=(float)get_total_pan() * mix_volume  / 127.0; //get_total_pan returns the pan to use, it goes from 0 to 127, so we'll do multipliers from there
	mix_volume_left=(float)( 127 - get_total_pan() ) * mix_volume / 127.0;

   	while (p_amount--) { //ok, the amount of SAMPLES to mix (remember, buffer is STEREO

		val = sin (offset); // <- here's the evil sine of death!!
		offset+=increment; //increment offset
		
		left = val * mix_volume_left;
		right = val * mix_volume_right;

                left*=REQUESTED_MAX_SAMPLE_VALUE; //this is the max value for the voice, it's actually (1<<29)
                right*=REQUESTED_MAX_SAMPLE_VALUE;
   
    		*p_where += left; //mix to channel buffer, left sample

  		*p_where++;

    		*p_where += right; //mix to channel buffer, right sample

  		*p_where++;
       	}

        store_last_values(left,right); //this goes to the declicker, to try avoid clicking if the voice is abruptly cut
}

SINE_Voice::SINE_Voice(){

	noteoff_was_pressed=false;
	base_freq=1;

	increment=0;
	offset=0;
	decay=1;
        

}
SINE_Voice::~SINE_Voice(){

}
