/***************************************************************************
                          bank.cpp  -  description
                             -------------------
    begin                : Fri Jul 5 2002
    copyright            : (C) 2002 by red
    email                : red@server
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "bank.h"

const int Bank::version = 2;


void Bank::get_patch_data_vector(int p_patch, vector<Uint8> &p_vector) {

        vector<Uint8> new_vector;

	append_string_to_vector("PTCH",new_vector);
	append_string_to_vector(get_model_name(),new_vector);
	append_int_to_vector(version,new_vector);
	append_string_to_vector(get_patch_name(p_patch),new_vector);

 	get_patch_properties(p_patch)->append_data_to_vector(new_vector);
  	
  	append_int_to_vector(new_vector.size(),p_vector);
	append_vector_to_vector(p_vector,new_vector);   	

}

Bank * Bank::get_drums(int p_index) {

	if ((p_index<0) || (p_index>=drums.size())) return NULL;

	return drums[p_index];
}

Voice * Bank::get_voice(int p_patch) {
	
	Voice *aux_voice=get_voice_internal(p_patch);
	aux_voice->set_default_data(&patch_defaults[p_patch]);
	return aux_voice;		
}

bool Bank::set_patch_data_vector(int p_patch, Uint8* p_vector,int p_size) { //true on error

	int ofs=0,ver;

 	string tmp_string;


 	ofs+=get_string_from_vector(tmp_string,p_vector);


  	if (tmp_string!="PTCH") {

           	ERROR("set_patch_data_vector: invalid patch format\n");
		return true;
   	}

 	ofs+=get_string_from_vector(tmp_string,&p_vector[ofs]);

  	if (tmp_string!=get_model_name()) {

           	ERROR("set_patch_data_vector: invalid patch model\n");
   		return true;
     	}

      	ver=get_int_from_vector(&p_vector[ofs]);

   	ofs+=4;
 	ofs+=get_string_from_vector(tmp_string,&p_vector[ofs]);

   	set_patch_name(p_patch,tmp_string);
    	
	get_patch_properties(p_patch)->read_data_from_array(&p_vector[ofs],p_size-ofs);
   	
   	return false;
}


void Bank::get_bank_data_vector(vector<Uint8> &p_vector) {

        vector<Uint8> new_vector;

	append_string_to_vector("BANK",new_vector);
	append_string_to_vector(get_model_name(),new_vector);

	append_int_to_vector(version,new_vector);
	append_string_to_vector(get_name(),new_vector);

 	append_int_to_vector(get_type(),new_vector);

   	if (get_type()==INSTRUMENT) { //save drums

  		append_int_to_vector(get_drums_count(),new_vector);
    
    		for (int i=0;i<get_drums_count();i++) {

			get_drums(i)->get_bank_data_vector(new_vector);
      		}
   	} else { //save drum notes


	 	for (int i=0;i<128;i++)
	  		append_int_to_vector(drum_note[i],new_vector);
    	}
     

 	for (int i=0;i<128;i++) {

		get_patch_data_vector(i,new_vector);		
  	}

  	append_int_to_vector(new_vector.size(),p_vector);
	append_vector_to_vector(p_vector,new_vector);   	

}


bool Bank::set_bank_data_vector(Uint8* p_vector,int p_size) {

	int ofs=0;
 	string tmp_string;
  	int ttype,ver;

 	ofs+=get_string_from_vector(tmp_string,p_vector);

  	if (tmp_string!="BANK") return true;

 	ofs+=get_string_from_vector(tmp_string,&p_vector[ofs]);

  	if (tmp_string!=get_model_name()) return true;

   	ver=get_int_from_vector(&p_vector[ofs]); //version

    	ofs+=4;

 	ofs+=get_string_from_vector(tmp_string,&p_vector[ofs]);
  	//name
   	set_name(tmp_string);

   	ttype=get_int_from_vector(&p_vector[ofs]);

 	ofs+=4;
  	set_type((Type)ttype);

   	if (ver>=2) { //version 1 doesnt support drums
	
	   	if (get_type()==INSTRUMENT) { //get_drums drums

	    		int drums_amount=get_int_from_vector(&p_vector[ofs]);
	      		ofs+=4;

	    		for (int i=0;i<drums_amount;i++) {

				int banksize=get_int_from_vector(&p_vector[ofs]);
				ofs+=4;
    				//get_drums(i)->get_bank_data_vector(new_vector);
    				add_drums();
        			if (get_drums(i)->set_bank_data_vector(&p_vector[ofs],banksize)) {

           				ERROR("Invalid drums bank format?");
               				return true;
               			}
           			ofs+=banksize;
	      		}
	   	} else { //load drum notes

		 	for (int i=0;i<128;i++) {
		  		drum_note[i]=get_int_from_vector(&p_vector[ofs]);
		    		ofs+=4;
		    	}
	    	}
	}	  	   

	for (int i=0;i<128;i++) {

         	int size=get_int_from_vector(&p_vector[ofs]);
         	ofs+=4;

		if (set_patch_data_vector(i,&p_vector[ofs],size)) {

          		return true;
            	}

    		ofs+=size;
  	}

  		
   	return false;


}

void Bank::add_drums() {

	drums.push_back(get_drums_bank());
	drums[drums.size()-1]->set_type(DRUM);
	for (int i=0;i<128;i++) {

		drums[drums.size()-1]->set_patch_name(i,"");
	}
	
	drums[drums.size()-1]->set_patch_name(0,"-- Standard GM drum layout --");
	drums[drums.size()-1]->set_patch_name(2,"(feel free to modify)");
	drums[drums.size()-1]->set_patch_name(3,"If you want Standard MIDIs");
	drums[drums.size()-1]->set_patch_name(4,"to play properly, then you should");
	drums[drums.size()-1]->set_patch_name(5,"respect these preset names.");
	
	drums[drums.size()-1]->set_patch_name(35,"Accent KickDrum");
	drums[drums.size()-1]->set_patch_name(36,"Normal KickDrum");
	drums[drums.size()-1]->set_patch_name(37,"Side WoodStick");
	drums[drums.size()-1]->set_patch_name(38,"Snare 1 (Ballad)");
	drums[drums.size()-1]->set_patch_name(39,"Hand Clap");
	drums[drums.size()-1]->set_patch_name(40,"Snare 2 (Short)");
	drums[drums.size()-1]->set_patch_name(41,"Extra Low Tom");
	drums[drums.size()-1]->set_patch_name(42,"Closed HiHat");
	drums[drums.size()-1]->set_patch_name(43,"Low Tom");
	drums[drums.size()-1]->set_patch_name(44,"Pedal HiHat");
	drums[drums.size()-1]->set_patch_name(45,"Medium Tom Low");
	drums[drums.size()-1]->set_patch_name(46,"Open HiHat");
	drums[drums.size()-1]->set_patch_name(47,"Medium Tom High");
	drums[drums.size()-1]->set_patch_name(48,"High Tom Low");
	drums[drums.size()-1]->set_patch_name(49,"Crash");
	drums[drums.size()-1]->set_patch_name(50,"High Tom High");
	drums[drums.size()-1]->set_patch_name(51,"Ride");
	drums[drums.size()-1]->set_patch_name(52,"Chinese Cymbal");
	drums[drums.size()-1]->set_patch_name(53,"Ride Bell");
	drums[drums.size()-1]->set_patch_name(54,"Tambourine");
	drums[drums.size()-1]->set_patch_name(55,"Crash 2");
	drums[drums.size()-1]->set_patch_name(56,"Cowbell");
	drums[drums.size()-1]->set_patch_name(57,"Crash 3");
	drums[drums.size()-1]->set_patch_name(58,"Slap");
	drums[drums.size()-1]->set_patch_name(59,"Ride 2");
	drums[drums.size()-1]->set_patch_name(60,"Bongo Hi");
	drums[drums.size()-1]->set_patch_name(61,"Bongo Low");
	drums[drums.size()-1]->set_patch_name(62,"Conga Mute");
	drums[drums.size()-1]->set_patch_name(63,"Conga Open High");
	drums[drums.size()-1]->set_patch_name(64,"Conga Open Low");
	drums[drums.size()-1]->set_patch_name(65,"Timbale");
	drums[drums.size()-1]->set_patch_name(66,"Timbale 2");
	drums[drums.size()-1]->set_patch_name(67,"Agogo");
	drums[drums.size()-1]->set_patch_name(68,"Agogo 2");
	drums[drums.size()-1]->set_patch_name(69,"Cabasa Up");
	drums[drums.size()-1]->set_patch_name(70,"Maracas");
	                                       
	drums[drums.size()-1]->set_name("Drum 7ank "+itostr(drums.size()));
}
void Bank::delete_drums(int p_index) {

	if ((p_index<0) || (p_index>=drums.size())) return;
	delete drums[p_index];
	drums.erase(drums.begin()+p_index);
}


int Bank::get_patch_default_controller_internal(int p_patch,int p_ctrl) {


}

bool Bank::load_file(string p_file, int p_offset=0) {}
bool Bank::load_patch(string p_file, int p_index=0) {}


int Bank::get_patch_default_controller(int p_patch,int p_ctrl) {

	int ctrl;

	switch (p_ctrl) {

		case 1: {

			ctrl=patch_defaults[p_patch].vibrato_depth;
		} break;
		case 5: {

			ctrl=patch_defaults[p_patch].portamento_time_coarse;
		} break;
		case 65: {

			ctrl=patch_defaults[p_patch].do_portamento?127:0;
		} break;
		case 7: {

			ctrl=patch_defaults[p_patch].main_volume;
		} break;
		case 10: {

			ctrl=patch_defaults[p_patch].channel_pan+64;
		} break;
		case 91: {

			ctrl=patch_defaults[p_patch].default_reverb;
		} break;
		case 93: {

			ctrl=patch_defaults[p_patch].default_chorus;
		} break;

		default: {

			ctrl=get_patch_default_controller_internal(p_patch,p_ctrl);
		}
	}

	return ctrl;
}


Bank::Bank(){

	name="New Bank";
	type=INSTRUMENT;
	drum_note.resize(128);
	patch_defaults.resize(128);
	patch_prop.resize(128);

	
	for (int i=0;i<128;i++) {

		drum_note[i]=69;
		
      		patch_prop[i].push_back(new Section_Property("Patch Global"));
      		patch_prop[i].push_back(new Separator_Property("Voice Defaults:"));
        
		patch_prop[i].push_back(new Value_Property("Pan (Relative): ",-64,63,&patch_defaults[i].relative_pan ));
      		patch_prop[i].push_back(new Value_Property("Sweep Delay: ",0,0x7F,&patch_defaults[i].sweep_delay));
      		patch_prop[i].push_back(new Value_Property("Sweep Rate: ",-64,63,&patch_defaults[i].sweep_value));

      		patch_prop[i].push_back(new Value_Property("LFO Type: ",0,0x2,&patch_defaults[i].LFO_type));
      		patch_prop[i].push_back(new Value_Property("LFO Speed: ",0,0x7F,&patch_defaults[i].LFO_speed));
      		patch_prop[i].push_back(new Value_Property("LFO Depth: ",0,0x7F,&patch_defaults[i].LFO_depth));
      		patch_prop[i].push_back(new Value_Property("LFO Delay: ",0,0x7F,&patch_defaults[i].LFO_delay));
        
      		patch_prop[i].push_back(new Separator_Property("Channel Defaults:"));

		patch_prop[i].push_back(new Value_Property("Pan (Global): ",-64,63,&patch_defaults[i].channel_pan ));
        
      		patch_prop[i].push_back(new Value_Property("Vibrato Speed: ",0,0x7F,&patch_defaults[i].vibrato_speed));
      		patch_prop[i].push_back(new Value_Property("Vibrato Depth: ",0,0x7F,&patch_defaults[i].vibrato_depth));

      		patch_prop[i].push_back(new Bool_Property("Enable Portamento: ",&patch_defaults[i].do_portamento));

      		patch_prop[i].push_back(new Value_Property("Portamento Coarse: ",0,0x7F,&patch_defaults[i].portamento_time_coarse));
      		patch_prop[i].push_back(new Value_Property("Portamento Fine: ",0,0x7F,&patch_defaults[i].portamento_time_fine));

      		patch_prop[i].push_back(new Separator_Property("Channel Effects:"));

      		patch_prop[i].push_back(new Value_Property("Default Reverb: ",0,0x7F,&patch_defaults[i].default_reverb));

      		patch_prop[i].push_back(new Value_Property("Default Chorus: ",0,0x7F,&patch_defaults[i].default_chorus));
        
 	}
	
}
Bank::~Bank(){

	while (get_drums_count()>0) delete_drums(0);
}
