/***************************************************************************
                          midi_event.cpp  -  description
                             -------------------
    begin                : Mon Feb 4 2002
    copyright            : (C) 2002 by Juan Linietsky
    email                : coding@reduz.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "midi_event.h"
		
void Midi_Event::on_note_kill(int p_chan,int p_note) {

	Channel::Voice_List::iterator I=channel[p_chan].voice_list.find(p_note);

 	if (I==channel[p_chan].voice_list.end()) {

          	ERROR("Fatal error!! killing a note i dont have?! (this is a bug!)");
        	return;
         }


	channel[p_chan].voice_list.erase(I);
}


void Midi_Event::note_on(char p_chan,char p_note,char p_velocity) {


        event_count++;

	Channel::Voice_List::iterator I;
        int chan;
        Bank *from_bank;
        Voice *new_voice;
        int drums_note;
	
        I=channel[p_chan].voice_list.find(p_note);

	from_bank=dev_manager->get_bank(channel[p_chan].bank);
 	
  	if (from_bank==NULL) { //the bank this channel is on doesnt exist?

   		return;
     	}

      	
        if (p_chan==drums_channel) { //drums channel!!

        	Bank *from_drum = from_bank->get_drums(channel[p_chan].patch);

         	if (from_drum==NULL) return; //this doesnt have drums

          
		new_voice=from_drum->get_voice(p_note); //odd, huh?
		drums_note=from_drum->get_drum_note(p_note);
        } else {
        
	      	new_voice=from_bank->get_voice(channel[p_chan].patch);
	}	 

	new_voice->set_channel(p_chan); //mixer needs this, so i'm asigning it before.

	if (I!=channel[p_chan].voice_list.end()) { //we have it!

		Voice *old_voice=channel[p_chan].voice_list[p_note];

  		//seek and destroy
	  	mixer->replace_voice(old_voice,new_voice);
		         					
  								
	} else {

	  	mixer->allocate_voice(new_voice);

	}
	

//	new_voice->set_data(&bank[ channel[p_chan].bank ].patch[ channel[p_chan].patch ] );

        if (p_chan==drums_channel) { //drums channel!!

		new_voice->set_note(drums_note,p_velocity);        
	} else {

		new_voice->set_note(p_note,p_velocity);
	}	

	new_voice->set_pan(channel[p_chan].pan);
	new_voice->set_main_volume(channel[p_chan].main_vol);
	new_voice->set_expression(channel[p_chan].expression);
	new_voice->set_pitch_bend(channel[p_chan].pitch_bend);
	new_voice->set_pitch_depth(channel[p_chan].bender_sens);
	new_voice->set_sustain(channel[p_chan].sustain);	

	//set default controller crap

	for (int i=0;i<128;i++) {

		if (channel[p_chan].default_custom_ctrl[i]!=UNUSED_CTRL)
			new_voice->set_controller(i,channel[p_chan].default_custom_ctrl[i]);
	}

 	new_voice->on_kill.connect(SigC::bind<int,int>(SigC::slot(this,&Midi_Event::on_note_kill),p_chan,p_note));

	channel[p_chan].voice_list[p_note]=new_voice;		
}

void Midi_Event::note_off(char p_chan,char p_note,char p_velocity) {

	event_count++;

	Channel::Voice_List::iterator I;
        int chan;
	
        I=channel[p_chan].voice_list.find(p_note);

	
	if (I==channel[p_chan].voice_list.end()) {

		printf("I dont have this note for noteoff\n");
		//we done have this note	
		return;	
	}
	
	I->second->set_note_off(p_velocity);

}


void Midi_Event::controller(char p_chan,char p_ctrl,char p_param) {

        Channel::Voice_List::iterator I;

        event_count++;

	switch (p_ctrl) {
		case 06: { // RPN data MSB
		
  			channel[p_chan].rpn.datamsb=p_param;
     			check_registered_parameter_number(p_chan);
  		} break;
		
		case 07: { //main volume
                        channel[p_chan].main_vol=p_param;
			for (I=channel[p_chan].voice_list.begin();I!=channel[p_chan].voice_list.end();I++) {

				I->second->set_main_volume(p_param);
			}
		
		} break;	
	
		case 10: {   //pan!
                        channel[p_chan].pan=p_param;
			for (I=channel[p_chan].voice_list.begin();I!=channel[p_chan].voice_list.end();I++) {

				I->second->set_pan(p_param);
			}
		
		} break;
		case 11: { // expression
                        channel[p_chan].expression=p_param;
			for (I=channel[p_chan].voice_list.begin();I!=channel[p_chan].voice_list.end();I++) {

				I->second->set_expression(p_param);
			}
		
		} break;
		case 32: { // Set Bank

  			channel[p_chan].bank=p_param;
  		} break;
		case 64: { // Sustain notes

			channel[p_chan].sustain=p_param?true:false;
			for (I=channel[p_chan].voice_list.begin();I!=channel[p_chan].voice_list.end();I++) {

				I->second->set_sustain(channel[p_chan].sustain);
			}

  		} break;
		
		case 91: { // reverb
		
  			channel[p_chan].effects.reverb=p_param;
     			channel[p_chan].effects.reverb*=2;
        		mixer->set_channel_reverb_send(p_chan,channel[p_chan].effects.reverb);

  		} break;
		case 93: { // chorus

  			channel[p_chan].effects.chorus=p_param;
     			channel[p_chan].effects.chorus*=2;
        		mixer->set_channel_chorus_send(p_chan,channel[p_chan].effects.chorus);

  		} break;
		case 100: { // expression
		
  			channel[p_chan].rpn.lsb=p_param;
  		} break;
		case 101: { // expression
		
  			channel[p_chan].rpn.msb=p_param;
  		} break;
		case 120: 
		case 121:
		case 123: {
		
			kill_chan_notes(p_chan);
		} break;

		default: { //unsupported controller? maybe the voice will like it

			for (I=channel[p_chan].voice_list.begin();I!=channel[p_chan].voice_list.end();I++) {

				if (I->second->set_controller(p_ctrl,p_param)) { // recognized!

					channel[p_chan].default_custom_ctrl[p_ctrl]=p_param;
				}
			}
		}
	}

}

void Midi_Event::pitch_bend(char p_chan,int p_val) {

	Channel::Voice_List::iterator I;

	channel[p_chan].pitch_bend=p_val;
	
	for (I=channel[p_chan].voice_list.begin();I!=channel[p_chan].voice_list.end();I++) {

		I->second->set_pitch_bend(channel[p_chan].pitch_bend);
		I->second->set_pitch_depth(channel[p_chan].bender_sens);
        }

}

void Midi_Event::program_change(char p_chan,char p_program) {


	kill_chan_notes(p_chan);
	if (channel[p_chan].patch==p_program) return;
        channel[p_chan].patch=p_program;

        //Get patch defaults
        
        Bank *sbank = dev_manager->get_bank(channel[p_chan].bank);

        if (sbank==NULL) return; //nothing to do

        channel[p_chan].pan=sbank->get_patch_default_controller(p_program,10);
        channel[p_chan].main_vol=sbank->get_patch_default_controller(p_program,7);
        channel[p_chan].effects.chorus=sbank->get_patch_default_controller(p_program,93);
        channel[p_chan].effects.reverb=sbank->get_patch_default_controller(p_program,91);
}

void Midi_Event::kill_chan_notes(char p_chan) {

	Channel::Voice_List::iterator I,J;

	for (I=channel[p_chan].voice_list.begin();I!=channel[p_chan].voice_list.end();) {
  	
          	J=I;
           	J++;



           	Voice *voice=I->second;
		
		delete voice;

		I=J;
        }

	for (int i=0;i<128;i++) channel[p_chan].default_custom_ctrl[i]=UNUSED_CTRL; //reset default ctrl

}

void Midi_Event::check_registered_parameter_number(int p_chan) {


 	if ((channel[p_chan].rpn.msb==0) && (channel[p_chan].rpn.lsb==0)) {
         	//standard stuff
		switch (channel[p_chan].rpn.datalsb) {

          		case 0: { //pitch bend range

				channel[p_chan].bender_sens=channel[p_chan].rpn.datamsb;

				for (Channel::Voice_List::iterator I=channel[p_chan].voice_list.begin();I!=channel[p_chan].voice_list.end();I++) {

					I->second->set_pitch_depth(channel[p_chan].rpn.datamsb);
				}

            		} break;
          	}        		
  	}

}

int Midi_Event::get_chan_bank(int p_chan) {


	return channel[p_chan].bank;
}
int Midi_Event::get_chan_patch(int p_chan) {

	return channel[p_chan].patch;

}


Midi_Event::Midi_Event(){

	event_count=0;
	drums_channel=9; //This is channel 10 in your sequencer
}
Midi_Event::~Midi_Event(){
}
