/***************************************************************************
                          midi_event.h  -  description
                             -------------------
    begin                : Mon Feb 4 2002
    copyright            : (C) 2002 by Juan Linietsky
    email                : coding@reduz.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MIDI_EVENT_H
#define MIDI_EVENT_H


#include "voice.h"
#include "mixer.h"
#include "device_manager.h"
#include <vector>
#include <hash_map>
/**
  *@author Juan Linietsky
  */

#define PAN_LEFT 0
#define PAN_RIGHT 127
#define PAN_CENTER 64


class Midi_Event : public SigC::Object {

public:
	Mixer *mixer;
	Device_Manager *dev_manager;
	void on_note_kill(int p_chan,int p_note);
private:	

	enum {

		UNUSED_CTRL=-1
	};
		
	struct Channel {
	
		int patch;
		int bank;
		int index;

		char default_custom_ctrl[128];
		
		typedef hash_map<int,Voice*> Voice_List;
	
		Voice_List voice_list;
		
		int pan;
		int main_vol;
		int expression;
		int pitch_bend;
		int bender_sens;

		bool sustain;
		
                struct Effects {

                        int reverb;
                        int chorus;

                } effects;

  		struct Rpn {

	
            		int msb,lsb;
              		int datamsb,datalsb;			
            	} rpn;

		Channel() {
		
			pan=PAN_CENTER;
			patch=0;
			bank=0;
			main_vol=127;
			expression=127;
			pitch_bend=0x2000;
			bender_sens=2;
   			rpn.lsb=0;
   			rpn.msb=0;
   			rpn.datamsb=0;
   			rpn.datalsb=0;
      			effects.reverb=0;
      			effects.chorus=0;
         		sustain=false;
           		for (int i=0;i<128;i++) default_custom_ctrl[i]=UNUSED_CTRL;
		};		
	};


        int drums_channel;
	
	Channel channel[16];
	
	void kill_chan_notes(char p_chan);	

	int event_count;	

  	void check_registered_parameter_number(int p_chan);
public:

	void set_drums_channel(int p_chan) { drums_channel=p_chan; }
	int get_drums_channel() { return drums_channel; }


	int get_chan_bank(int p_chan);
	int get_chan_patch(int p_chan);

        int get_event_count() { return event_count; }


	void note_on(char p_chan,char p_note,char p_velocity);
	void note_off(char p_chan,char p_note,char p_velocity);
	void controller(char p_chan,char p_ctrl,char p_param);
	void program_change(char p_chan,char p_program);
 	void pitch_bend(char p_chan,int p_val);
	

  	
	Midi_Event();
	~Midi_Event();
};

#endif
