/***************************************************************************
                          mixer.cpp  -  description
                             -------------------
    begin                : Sun Jan 27 2002
    copyright            : (C) 2002 by Juan Linietsky
    email                : coding@reduz.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mixer.h"
#include "math.h"

static double ofs =0;

void Mixer::add_send_buffer(Sint32* p_dst, Sint32 *p_src, int p_amount,int p_percent /* 0 - 0xFF */) {

	while (p_amount--) {

         	*p_dst++ += ((*p_src >> 8) * p_percent); //Left
          	p_src++;
         	*p_dst++ += ((*p_src >> 8) * p_percent);; //Right
          	p_src++;

           	//quality loss my ass, you cant notice it anyway
	}        	
}


void Mixer::add_buffer(Sint32* p_dst, Sint32 *p_src, int p_amount) {

	while (p_amount--) {

         	*p_dst++ += *p_src; //Left
          	p_src++;
         	*p_dst++ += *p_src; //Right
          	p_src++;
	}        	
}

int Mixer::write_bytes(void *p_buff,int bytes) {


 	int buffer_size=mix_stereo?(MIX_BUFFER_SIZE/2):MIX_BUFFER_SIZE;
        int aux_active_voices=0;
        //printf("DO STUFF!\n");


	if (bytes<buffer_size) buffer_size=bytes;



 	memset(&mix_buffer[0],0,buffer_size*sizeof(Sint32)*2); //clear buffer
 	memset(&reverb_buffer[0],0,buffer_size*sizeof(Sint32)*2); //clear buffer
 	memset(&chorus_buffer[0],0,buffer_size*sizeof(Sint32)*2); //clear buffer

   	for (int i=0;i<16;i++) {

		channel[i].used_in_frame=false; //no channel used by default
    	} 	

	for(int i=0;i<voice.size();i++) {

         	//if this voice doesnt exist (NULL) then we dont bother
         	if (voice[i]==NULL) continue;

          	int chan; //check chan

           	chan=voice[i]->get_channel();

    		voice[i]->set_preamp(preamp); //pre volume amplification before mixing

            	if (!channel[chan].used_in_frame) { //if this channel wasnt used in this frame..

			//we zero it
			memset(&channel[chan].buffer[0],0,buffer_size*sizeof(Sint32)*2);
   			channel[chan].used_in_frame=true; //and we set it as "used in frame"
      			
             	}

		voice[i]->mix(buffer_size,&channel[chan].buffer[0]); //and we mix to it..

  		voice[i]->poll_for_dead_staus(); //now btw let's ask the voice to figure out if it's actually doing something

		if (voice[i]->get_status()==Voice::DEAD) { //if it doesnt, murder it

			declicker_list.push_back(voice[i]->get_last_value_L());
			declicker_list.push_back(voice[i]->get_last_value_R());
		
          		delete voice[i]; //bloodily
            	
  			continue; //and walk away. We didnt see a thing, word!
     		}


  		aux_active_voices++; //otherwise, must sum to the active voices count
  	}

/*
	for(int i=0;i<bytes;i++) {

		double val=sin((ofs/44100)*3.1415);
  		val *=(1<<30);
  		ofs++;
    		mix_buffer[i*2]=val;
    		mix_buffer[i*2+1]=val;
  		
	}
  */
        //This mixes back the voices into the main buffer and effect buffers... only if they were used

   	for (int i=0;i<16;i++) {

		if (channel[i].used_in_frame) {

			add_buffer(&mix_buffer[0],&channel[i].buffer[0],buffer_size);
   			add_send_buffer(&reverb_buffer[0],&channel[i].buffer[0],buffer_size,channel[i].effects.reverb_send);
   			add_send_buffer(&chorus_buffer[0],&channel[i].buffer[0],buffer_size,channel[i].effects.chorus_send);
  		}

    		channel[i].last_used_in_frame=channel[i].used_in_frame;
    	}

   	active_voices=aux_active_voices;

        process_reverb(buffer_size);
        process_chorus(buffer_size);
	process_declicker();

        if (dont_mix_back)
       		return buffer_size; //just.. dont mix back! ;)

   	if (mix_16bits) {

            	if (mix_stereo) {

			write_buffer_bytes_16_stereo(p_buff,buffer_size);
             	} else {

			write_buffer_bytes_16_mono(p_buff,buffer_size);
		}

    	} else {

            	if (mix_stereo) {


			write_buffer_bytes_8_stereo(p_buff,buffer_size);
             	} else {

			write_buffer_bytes_8_mono(p_buff,buffer_size);
		}

     	}
   	/*
  	  */
	return buffer_size;
}	
	
void Mixer::on_voice_kill(int p_index) {

	voice[p_index]=NULL;
}

void Mixer::write_buffer_bytes_8_mono(void *p_buf,int bytes) {

        int amount=bytes;
	Uint8 *buff8=((Uint8*)p_buf);	
 	Sint32 *src = &mix_buffer[0];
	int thing;
		
	while (amount>0) {
	
		thing=*src/2;
  		src++;
  		thing+=*src/2;
  		src++;
                thing>>=24; //yay
		*buff8=thing;
		buff8++;
		amount--;		
	}
}

void Mixer::write_buffer_bytes_8_stereo(void *p_buf,int bytes) {

        int amount=bytes;
	Uint8 *buff8=((Uint8*)p_buf);	
 	Sint32 *src = &mix_buffer[0];
	int thing;
		
	while (amount--) {
	
		thing=*src >> 24;
  		src++;
		*buff8++ =thing;

		thing=*src >> 24;
  		src++;
		*buff8++ =thing;

	}


}

void Mixer::write_buffer_bytes_16_mono(void *p_buf,int bytes) {

        int amount=bytes;
	Sint16 *buff16=((Sint16*)p_buf);	
 	Sint32 *src = &mix_buffer[0];
	int thing;
		
	while (amount--) {
	
		thing=*src/2;
  		src++;
  		thing+=*src/2;
  		src++;

                thing>>=16; //yay
		*buff16++ =thing;
	}

}

void Mixer::write_buffer_bytes_16_stereo(void *p_buf,int bytes) {

        int amount=bytes;
	Sint16 *buff16=((Sint16*)p_buf);	
 	Sint32 *src = &mix_buffer[0];
	int thing;
		
	while (amount--) {
	
		*buff16++ =*src >> 16;
  		src++;
		*buff16++ =*src >> 16;
  		src++;
	}
}

void Mixer::replace_voice(Voice* p_old_voice,Voice* p_new_voice) {


	for (int i=0;i<voice.size();i++) {

		if (voice[i]==p_old_voice) {

          		delete voice[i];
			voice[i]=p_new_voice;
			p_new_voice->set_mix_frequency(mix_freq);
   			voice[i]->on_kill.connect(SigC::bind<int>(SigC::slot(this,&Mixer::on_voice_kill),i));
  			return;
          	}
 	}

  	ERROR("Trying to replace an inexistent voice?");
	allocate_voice(p_new_voice); //in case we're trying to replace an inexisten voice?		
}
		
void Mixer::allocate_voice(Voice* p_new_voice) {

	int where=find_aviable_voice();
	voice[where]=p_new_voice;
        p_new_voice->set_mix_frequency(mix_freq);
	p_new_voice->on_kill.connect(SigC::bind<int>(SigC::slot(this,&Mixer::on_voice_kill),where));

}


int Mixer::find_aviable_voice() {

        int i;

	for(i=0;i<voice.size();i++) { //first let's see if there's something unused

 		if (voice[i]==NULL) return i;
   	}

	for(i=0;i<voice.size();i++) {

 		if (voice[i]->get_status()==Voice::DEAD) {

   			declicker_list.push_back(voice[i]->get_last_value_L());
   			declicker_list.push_back(voice[i]->get_last_value_R());
      
           		delete voice[i];
    			return i;
       		}
   	}

	declicker_list.push_back(voice[0]->get_last_value_L());
	declicker_list.push_back(voice[0]->get_last_value_R());


	return 0;
}


void Mixer::set_reverb_params(float p_room_size,float p_damp,float p_wet,float p_dry,float p_width) {


	reverb.room_size=p_room_size*3;
	reverb.damp=p_damp*3;
	reverb.wet=p_wet*3;
	reverb.dry=p_dry*3;
	reverb.width=p_width*3;
	
        freeverb.setroomsize(p_room_size);
        freeverb.setdamp(p_damp);
	freeverb.setwet(p_wet);
	freeverb.setdry(p_dry);
	freeverb.setwidth(p_width);
	freeverb.setmode(0);
}

void Mixer::get_reverb_params(float &p_room_size,float &p_damp,float &p_wet,float &p_dry,float &p_width) {

	p_room_size=reverb.room_size/3;
	p_damp=reverb.damp/3;
	p_wet=reverb.wet/3;
	p_dry=reverb.dry/3;
	p_width=reverb.width/3;

}

void Mixer::set_chorus_params(float p_delay, float p_lfo_speed, float p_lfo_depth, float p_feedback, float p_width) {

	chorus.set_chorus_params(p_delay,p_lfo_speed,p_lfo_depth,p_feedback,p_width);


}

void Mixer::get_chorus_params(float &p_delay, float &p_lfo_speed, float &p_lfo_depth, float &p_feedback, float &p_width) {

	chorus.get_chorus_params(p_delay,p_lfo_speed,p_lfo_depth,p_feedback,p_width);

}
void Mixer::set_dont_mix_back(bool p_mixback) {

	dont_mix_back=p_mixback;
}
void Mixer::set_preamp(float p_preamp) {

	preamp=p_preamp;
}

Mixer::Mixer() : chorus(MIX_BUFFER_SIZE*4) {

	mix_freq=0;

	dont_mix_back=false;

	set_reverb_params(0.7,0.8,0.65,0.1,1.0);
	freeverb.setmode(0);
	
        voice.resize(32);
	for (int i=0;i<voice.size();i++) {

         	voice[i]=NULL;
 	}

  	channel.resize(16);

   	for (int i=0;i<16;i++) {

		channel[i].buffer.resize(MIX_BUFFER_SIZE*2);
    	}
	mix_buffer.resize(MIX_BUFFER_SIZE*2);

	reverb_buffer.resize(MIX_BUFFER_SIZE*4);
	chorus_buffer.resize(MIX_BUFFER_SIZE*4);

 	reverb_in_l.resize(MIX_BUFFER_SIZE*2);
 	reverb_in_r.resize(MIX_BUFFER_SIZE*2);
 	reverb_out_l.resize(MIX_BUFFER_SIZE*2);
 	reverb_out_r.resize(MIX_BUFFER_SIZE*2);
  	preamp=1;
}


Mixer::~Mixer(){
}
