/***************************************************************************
                          mixer.h  -  description
                             -------------------
    begin                : Sun Jan 27 2002
    copyright            : (C) 2002 by Juan Linietsky
    email                : coding@reduz.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MIXER_H
#define MIXER_H

#include "revmodel.h"

#include "voice.h"
#include "chorus.h"

#include <vector>
/**
  *@author Juan Linietsky
  */

#define MIX_BUFFER_SIZE 30000

#define DECLICKER_BUFFER_SIZE 20
#define DECLICKER_THRESHOLD (1<<18) //if higher than this, might click i guess

class Mixer : public SigC::Object {

	int mix_freq;
	bool mix_16bits;
	bool mix_stereo;
	bool p_big_endian;


	void write_buffer_bytes_8_mono(void *p_buf,int bytes);
	void write_buffer_bytes_8_stereo(void *p_buf,int bytes);	
	void write_buffer_bytes_16_mono(void *p_buf,int bytes);
	void write_buffer_bytes_16_stereo(void *p_buf,int bytes);	

 	vector<Voice*> voice;	

  	vector<Sint32> mix_buffer;
	vector<Sint32> declicker_list;
   	

   	struct Channel {

		vector<Sint32> buffer;
  		bool used_in_frame; //if it wasnt used, avoids overhead
  		bool last_used_in_frame; //if it wasnt used, avoids overhead

    		struct Effects {

			int reverb_send;// 0 - 0xFF
			int chorus_send;// 0 - 0xFF

              	} effects;

    		Channel() {
                        effects.chorus_send=80;
                        effects.reverb_send=80;
			last_used_in_frame=used_in_frame=false;

   			
   		}
    	};

  	vector<Channel> channel;

   	void add_buffer(Sint32* p_dst, Sint32 *p_src, int p_amount);
   	void add_send_buffer(Sint32* p_dst, Sint32 *p_src, int p_amount,int p_percent /* 0 - 0xFF */);
   	

   	int active_voices;

	void on_voice_kill(int p_index);

        /* Effect extras */

	revmodel freeverb; //freeverb object
	Chorus chorus; //chorus object
  	vector<Sint32> reverb_buffer;
  	vector<Sint32> chorus_buffer;
   
   	vector<float> reverb_in_l; //freeverb needs those
	vector<float> reverb_in_r;
	vector<float> reverb_out_l;
	vector<float> reverb_out_r;

        struct Reverb {

		float room_size;
		float damp;
		float wet;
		float dry;
		float width;

		Reverb() {

			memset(this,0,sizeof(Reverb));
		}

	} reverb;
	
        float preamp;
        
        void process_chorus(int p_amount);
        void process_reverb(int p_amount);
        void process_declicker();

        bool dont_mix_back;

public:

        Sint32 *get_mix_buf_ptr() { return &mix_buffer[0]; }

        Sint32 *get_buf_ptr(int p_chan) { return &channel[p_chan].buffer[0]; }
	bool is_channel_in_use(int p_chan) { return channel[p_chan].last_used_in_frame; };		

 	void set_channel_reverb_send(int p_chan,Uint8 p_amount) { channel[p_chan].effects.reverb_send=p_amount; };
 	void set_channel_chorus_send(int p_chan,Uint8 p_amount) { channel[p_chan].effects.chorus_send=p_amount; };

	int get_active_voices() { return active_voices; };		

	
	void allocate_voice(Voice* p_new_voice);
	void replace_voice(Voice* p_old_voice,Voice* p_new_voice);

//        void set_voice
	void set_mix_frequency(int p_freq) {mix_freq=p_freq;}
	void set_mix_stereo(bool p_stereo) {mix_stereo=p_stereo;}
	void set_mix_16bits(bool p_16bits) {mix_16bits=p_16bits;}

        int get_mix_frequency() { return mix_freq; };
	
        void set_dont_mix_back(bool p_mixback); //dont mix to the ptr parameter, the audio driver will mix for me in its own format (damned jack ;)

	void set_chorus_params(float p_delay, float p_lfo_speed, float p_lfo_depth, float p_feedback, float p_width);
        void get_chorus_params(float &p_delay, float &p_lfo_speed, float &p_lfo_depth, float &p_feedback, float &p_width);

	void set_reverb_params(float p_room_size,float p_damp,float p_wet,float p_dry,float p_width);
	void get_reverb_params(float &p_room_size,float &p_damp,float &p_wet,float &p_dry,float &p_width);

        void set_preamp(float p_preamp);
	
	int write_bytes(void *p_buf,int bytes);
	
	int find_aviable_voice();
	
	Mixer();
	~Mixer();
};

#endif
