/***************************************************************************
                          property.h  -  description
                             -------------------
    begin                : Fri Jul 5 2002
    copyright            : (C) 2002 by red
    email                : red@server
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PROPERTY_H
#define PROPERTY_H

#include <typedefs.h>
#include <vector>
#include <list>
/**
  *@author red
  */


/* helper crap for data managing */


void append_int_to_vector(int p_val,vector<Uint8> &p_data);
void append_string_to_vector(string p_string,vector<Uint8> &p_data);
void append_vector_to_vector(vector<Uint8> &p_dest,vector<Uint8> &p_src);
unsigned get_int_from_vector(Uint8 *p_vector);
unsigned get_string_from_vector(string &p_string,Uint8 *p_vector);

bool load_vector_from_file(string p_filename,vector<Uint8> &p_data);
bool save_vector_to_file(string p_filename,vector<Uint8> &p_data);


class Property {
protected:

	string type; //property type
 	string name; //property name
public:

	string get_type() { return type; };
	string get_name() { return name; };

	Property();
	~Property();
};

class PropertyList { //i dont know why i dont want to do just a typedef


	list<Property*> prop_list;
public:

	typedef list<Property*>::iterator iterator;
	list<Property*>::iterator begin() { return prop_list.begin(); }
	list<Property*>::iterator end() { return prop_list.end(); }

 	void push_back(Property* p_new_property) {

     		prop_list.push_back(p_new_property);
    	}

     	void clear() {

         	for (iterator I=begin();I!=end();I++) {
          		delete (*I);
            	}

             	prop_list.clear();
	}

 	void append_data_to_vector(vector<Uint8> &p_data);
	void read_data_from_array(Uint8 *p_data,int p_size);

	~PropertyList() {
	
         	clear();	
	}
};
/* basic/common ones */

class Value_Property : public Property {

        int min,max;
        int *val;

public:
	int get_min() { return min; }
	int get_max() { return max; }
	int get_val() { return *val; }
 	void set_val(int p_val) { *val=(p_val<min)?min:((p_val>max)?max:p_val); };
        Value_Property(string p_name,int p_min,int p_max,int *p_val) {name=p_name; type="value"; min=p_min; max=p_max; val=p_val; };
}; 	

class Bool_Property : public Property {

        bool *status;
public:
	bool get_status() { return *status; }
 	void set_status(bool p_status) { *status = p_status; };
        Bool_Property(string p_name, bool *p_status) { name=p_name; type="bool"; status=p_status; }
}; 	

class Multiple_Choice_Property : public Property {

	vector<string> choices;
 	int *selected;
public:
	int get_choice_count() { return choices.size(); }
	string get_choice(int p_index) { return choices[p_index]; }
	int get_selected() { return *selected; }
 	void set_selected(int p_selected) { *selected= (p_selected<0) ? 0: ( (p_selected>=choices.size()) ? (choices.size()-1) : p_selected );};
	Multiple_Choice_Property(string p_name, vector<string> &p_choices, int *p_selected) { name=p_name; type="option"; selected=p_selected; choices=p_choices; }
}; 	

class Image_Choice_Property : public Property {

	vector<char*> choices;
 	int *selected;
public:
	int get_choice_count() { return choices.size(); }
	char* get_choice(int p_index) { return choices[p_index]; }
	int get_selected() { return *selected; }
 	void set_selected(int p_selected) { *selected= (p_selected<0) ? 0: ( (p_selected>=choices.size()) ? (choices.size()-1) : p_selected );};
	Image_Choice_Property(string p_name, vector<char*> &p_choices, int *p_selected) { name=p_name; type="image_option"; selected=p_selected; choices=p_choices; }
};

class Separator_Property : public Property {

public:
        Separator_Property(string p_name) { name=p_name; type="separator"; }
}; 	

class Section_Property : public Property {

public:
        Section_Property(string p_name) { name=p_name; type="section"; }
}; 	


#endif
