/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Mon Jan  7 13:44:38 ART 2002
    copyright            : (C) 2002 by Juan Linietsky
    email                : coding@reduz.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "sound_driver_oss.h"
#include "sound_driver_win32.h"
#include "sound_driver_jack.h"
#include "alsa_client.h"
#include "win32_client.h"


#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#include <gtk--/main.h>
#include "interface.h"
#include "device_manager.h"
#include "sound_driver_manager.h"
#include "audio_manager.h"

#include "dx7_device.h"
#include "sid_device.h"
#include "ym2x_devoce.h"
#include "303_device.h"
#include "sine_device.h"

#ifdef WIN32_ENABLED

#include "mutex_lock_win32"

Sound_Driver_Win32 driver;
Win32_Client client;
#define SLEEP_FUNCTION Sleep

Mutex_Lock_Win32 global_mutex;

#else

#ifdef JACK_ENABLED

Sound_Driver_JACK driver_jack;

#endif;

#include "mutex_lock_pthreads.h"

Mutex_Lock_Pthreads global_mutex;

Sound_Driver_OSS driver;
Alsa_Client client;

pthread_t player_thread;
pthread_attr_t thread_attr;
sched_param thread_sched_param;

#define SLEEP_FUNCTION usleep

#endif


Audio_Manager audio_manager;

bool thread_quit=false;

void * thread_function(void *something) {

        while (!thread_quit) {

		audio_manager.external_update();	
		SLEEP_FUNCTION(1);
  	}
}


int main(int argc, char *argv[]) {


	bool quit=false;
 	int counter=0;
        Gtk::Main kit(argc, argv);
	Device_Manager device_manager;
        Interface interface;

        Dx7_Device dx7_device;
	SID_Device sid_device;
	OPN_Device opn_device;
	TB303_Device tb303_device;
	SINE_Device sine_device;
	Midi_Event midi_event;
 	Mixer mixer;

  	audio_manager.driver_manager.register_driver(&driver);
        audio_manager.driver_manager.set_variables_lock(&global_mutex);
        
        device_manager.add_device(&dx7_device);
        device_manager.add_device(&sid_device);
        device_manager.add_device(&opn_device);
        device_manager.add_device(&tb303_device);
        
#ifdef SINE_ENABLED
        device_manager.add_device(&sine_device);
#endif
        interface.set_device_manager(&device_manager);
        interface.show();
        interface.kit=&kit;
        interface.mutex=&global_mutex;
        interface.mixer=&mixer;
        interface.midi_event=&midi_event;
        interface.driver_manager=&audio_manager.driver_manager;

        audio_manager.midi_client=&client;
        client.midi_event=&midi_event;
        midi_event.mixer=&mixer;
        midi_event.dev_manager=&device_manager;
        driver.link_mixer(&mixer);

               
#ifdef JACK_ENABLED

	driver_jack.link_mixer(&mixer);
  	audio_manager.driver_manager.register_driver(&driver_jack);
#endif

        client.create_port();

	audio_manager.driver_manager.request_mix_frequency(44100);
	audio_manager.driver_manager.request_mix_stereo(true);
	audio_manager.driver_manager.request_mix_16bits(true);
	audio_manager.driver_manager.request_buffer_size(2048);

 	audio_manager.driver_manager.set_active_driver(0); //driver.init();
	audio_manager.driver_manager.reset_active_driver();
//  	printf("%i %i %i %i\n",driver_manager.get_mix_frequency(),driver_manager.get_mix_stereo(),driver_manager.get_mix_16bits(),driver_manager.get_mix_buffer_size());
  
	interface.interface_update_config();

	printf("Starting audio thread...\n");  	

  	{
                pthread_attr_init(&thread_attr);
		thread_sched_param.sched_priority=0;
		pthread_attr_setschedparam (&thread_attr,&thread_sched_param);
		pthread_create(&player_thread,&thread_attr,&thread_function,NULL);
   	}


 	printf("Initialization complete... LegaSynth running!\n");

	while (!quit) {


		while (kit.events_pending()) kit.iteration();
		if ((counter%4)==0) {
		
			interface.update_info_widgets();

		}
		counter++;
		SLEEP_FUNCTION(1);
	 	quit=interface.is_quit_requested();
		//printf("count %i\n",count);
	}
/*	
	driver.finish();
	delete config_file;
*/	  	
  	return EXIT_SUCCESS;
}
